/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.fleet.wizard;

import nl.cloudfarming.client.fleet.model.Fleet;
import nl.cloudfarming.client.fleet.model.FleetManager;
import java.awt.Component;
import java.io.File;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.LinkedHashSet;
import java.util.NoSuchElementException;
import java.util.Set;
import javax.swing.JComponent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.fleet.FleetProjectFactory;
import org.netbeans.api.templates.TemplateRegistration;
import org.netbeans.spi.project.ui.support.ProjectChooser;
import org.openide.WizardDescriptor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 *
 * @author Wytse Visser
 */
@TemplateRegistration(folder = "Project/Fleet", displayName = "#Fleet_displayName", description = "FleetDescription.html", iconBase = "nl/cloudfarming/client/fleet/fleet.png", content = "FleetProject.zip")
@Messages({"Fleet_displayName=Fleet"})
public class FleetWizardIterator implements WizardDescriptor.InstantiatingIterator {

    private int index;
    private WizardDescriptor.Panel[] panels;
    private WizardDescriptor wiz;
    public static final String PROP_PROJECT_NAME = "projectName";
    public static final String PROP_PROJECT_DIR = "projectDir";

    @Override
    public Set instantiate() throws IOException {
        Set<FileObject> createdProjectDirectories = new LinkedHashSet<>();

        FileObject projectDirectory = createProjectDirectoryFromWizardInput(PROP_PROJECT_DIR);
        Fleet fleet = createFleetFromWizardInput(PROP_PROJECT_NAME);
        saveNewFleet(fleet, projectDirectory);

        // Always open top dir as a project:
        createdProjectDirectories.add(projectDirectory);

        File parent = FileUtil.toFile(projectDirectory.getParent());
        if (parent != null
                && parent.exists()) {
            ProjectChooser.setProjectsFolder(parent);
        }

        return createdProjectDirectories;
    }

    private FileObject createProjectDirectoryFromWizardInput(String property) {
        File projectDirAsFile = FileUtil.normalizeFile((File) wiz.getProperty(property));
        projectDirAsFile.mkdirs();
        return FileUtil.toFileObject(projectDirAsFile);
    }

    private Fleet createFleetFromWizardInput(String property) {
        Fleet fleet = new Fleet();
        String fleetName = (String) wiz.getProperty(property);
        fleet.setName(fleetName);
        return fleet;
    }

    private void saveNewFleet(Fleet fleet, FileObject projectDirectory) throws IOException {
        Lookup.getDefault().lookup(FleetManager.class).save(fleet, projectDirectory.createData(FleetProjectFactory.FLEET_FILE));
    }

    private String[] createSteps() {
        return new String[]{
                    NbBundle.getMessage(FleetWizardIterator.class, "LBL_CreateProjectStep"),};
    }

    private WizardDescriptor.Panel[] createPanels() {
        return new WizardDescriptor.Panel[]{
                    new FleetWizardNamePanel()
                };
    }

    @Override
    public void initialize(WizardDescriptor wiz) {
        this.wiz = wiz;
        index = 0;
        panels = createPanels();

        // Make sure list of steps is accurate.
        String[] steps = createSteps();
        for (int i = 0; i < panels.length; i++) {
            Component c = panels[i].getComponent();
            if (steps[i] == null) {
                // Default step name to component name of panel.
                // Mainly useful for getting the name of the target
                // chooser to appear in the list of steps.
                steps[i] = c.getName();
            }
            if (c instanceof JComponent) { // assume Swing components
                JComponent jc = (JComponent) c;
                // Step #.
                // TODO if using org.openide.dialogs >= 7.8, can use WizardDescriptor.PROP_*:
                jc.putClientProperty("WizardPanel_contentSelectedIndex", new Integer(i));
                // Step name (actually the whole list for reference).
                jc.putClientProperty("WizardPanel_contentData", steps);
            }
        }
    }

    @Override
    public void uninitialize(WizardDescriptor wd) {
        this.wiz.putProperty(PROP_PROJECT_DIR, null);
        this.wiz.putProperty(PROP_PROJECT_NAME, null);
        this.wiz = null;
        panels = null;
    }

    @Override
    public WizardDescriptor.Panel current() {
        return panels[index];
    }

    @Override
    public String name() {
        return MessageFormat.format("{0} of {1}",
                new Object[]{new Integer(index + 1), new Integer(panels.length)});
    }

    @Override
    public boolean hasNext() {
        return index < panels.length - 1;
    }

    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    @Override
    public void nextPanel() {
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
    }

    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
    }

    @Override
    public void addChangeListener(ChangeListener cl) {
    }

    @Override
    public void removeChangeListener(ChangeListener cl) {
    }
}
